# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

import moksha.common.utils

import tw2.core as twc
import tw2.jquery

from moksha.wsgi.widgets.api.socket import AbstractMokshaSocket

orbited_host = twc.Required
orbited_port = twc.Required
orbited_scheme = twc.Required

stomp_broker = twc.Required
stomp_port = twc.Required
stomp_user = twc.Required
stomp_pass = twc.Required



def stomp_subscribe(topic):
    """ Return a javascript callback that subscribes to a given topic,
        or a list of topics.
    """
    sub = "stomp.subscribe('%s');"
    if isinstance(topic, list):
        sub = ''.join([sub % t for t in topic])
    else:
        sub = sub % topic
    return sub


def stomp_unsubscribe(topic):
    """ Return a javascript callback that unsubscribes to a given topic,
        or a list of topics.
    """
    sub = "stomp.unsubscribe('%s');"
    if isinstance(topic, list):
        sub = ''.join([sub % t for t in topic])
    else:
        sub = sub % topic
    return sub


class StompWidget(AbstractMokshaSocket):
    __shorthand__ = 'STOMP Socket'

    orbited_host = twc.Param(default=orbited_host)
    orbited_port = twc.Param(default=orbited_port)
    orbited_scheme = twc.Param(default=orbited_scheme)

    stomp_broker = twc.Param(default=stomp_broker)
    stomp_port = twc.Param(default=stomp_port)
    stomp_user = twc.Param(default=stomp_user)
    stomp_pass = twc.Param(default=stomp_pass)

    template = "mako:moksha.wsgi.widgets.api.stomp.templates.stomp"

    def prepare(self):
        super(StompWidget, self).prepare()
        self.orbited_url = '%s://%s:%s' % (
            self.orbited_scheme,
            self.orbited_host,
            self.orbited_port,
        )

        orbited_js = twc.JSLink(
            link=self.orbited_url + '/static/Orbited.js',
            resources=[tw2.jquery.jquery_js])

        tstomp_js = twc.JSLink(
            link=self.orbited_url + '/static/protocols/stomp/stomp.js')

        self.resources.extend([orbited_js, tstomp_js])




import webtest

import moksha.common.testtools.utils as testutils

import moksha.wsgi.widgets.api.live

from moksha.wsgi.middleware import make_moksha_middleware
from tw2.core import make_middleware as make_tw2_middleware


class TestLiveWidget:
    def _setUp(self):
        def kernel(config):
            def app(environ, start_response):
                start_response('200 OK', [('Content-Type', 'text/html')])
                class MyWidget(moksha.wsgi.widgets.api.live.LiveWidget):
                    template = "moksha.wsgi.tests.templates.empty"

                    onmessage = 'console.log(json)'
                    topic = 'test'

                    backend = config['moksha.livesocket.backend']

                return map(str, [MyWidget.display()])

            app = make_moksha_middleware(app, config)
            app = make_tw2_middleware(app, config)
            app = webtest.TestApp(app)
            self.app = app

        for _setup, name in testutils.make_setup_functions(kernel):
            yield _setup, name

    def _tearDown(self):
        pass

    @testutils.crosstest
    def test_can_render(self):
        response = self.app.get('/')



# Authors: Ralph Bean <rbean@redhat.com>

from moksha.wsgi.widgets.api import LiveWidget
from tw2.jit import AreaChart


class LiveAreaChartWidget(AreaChart, LiveWidget):
    """ A live graphing widget using tw2.jit """
    onmessage = 'window._jitwidgets["${id}"].loadJSON(json[0])'



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A module that imports all known widgets.

This is currently used by the `archive_tw_resources` setuptools command,
which archives all of the resources used by all ToscaWidgets.
"""

import pkg_resources

from tw2.core.widgets import WidgetMeta
from inspect import isclass

__all__ = []

for entry_point in ('moksha.widget', 'moksha.menu', 'moksha.global'):
    for widget_entry in pkg_resources.iter_entry_points(entry_point):
        widget_class = widget_entry.load()
        if isclass(widget_class) and not isinstance(widget_class, WidgetMeta):
            widget = widget_class(widget_entry.name)
        else:
            widget = widget_class
        __all__.append(widget)

# Other stuff that isn't exposed
from moksha.wsgi.widgets.container import MokshaContainer
__all__.append(MokshaContainer)



from tw2.core import js_callback


def when_ready(func):
    """
    Takes a js_function and returns a js_callback that will run
    when the document is ready.

        >>> from tw.api import js_function
        >>> print when_ready(js_function('jQuery')('foo').bar({'biz': 'baz'}))
        $(document).ready(function(){jQuery("foo").bar({"biz": "baz"})});
    """
    from tw2.core import js_callback
    return js_callback('$(document).ready(function(){' + str(func) + '});')



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

import os
import moksha.common.utils
import logging
import pkg_resources
import warnings
import types

from collections import defaultdict
from paste.deploy.converters import asbool
from inspect import isclass
from sqlalchemy import create_engine

from moksha.common.exc import MokshaException
from moksha.common.lib.helpers import get_moksha_config_path
from moksha.common.lib.helpers import appconfig

log = logging.getLogger(__name__)

# A list of all the entry points
APPS = 'moksha.application'
WIDGETS = 'moksha.widget'
ROOT = 'moksha.root'
MENUS = 'moksha.menu'


class MokshaMiddleware(object):
    """
    A layer of WSGI middleware that is responsible for setting up the moksha
    environment, as well as handling every request/response in the application.

    If a request for an application comes in (/apps/$NAME), it will dispatch to
    the WSGI Application or RootController of that application as defined in
    it's egg-info entry-points.

    This middleware also sets up the `moksha.livewidgets` StackedObjectProxy,
    which acts as a registry for Moksha LiveWidget topic callbacks.

    """
    def __init__(self, application, config):
        log.info('Creating Moksha Middleware')
        self.application = application
        self.config = config

        moksha.common.utils._apps = {}
        moksha.common.utils._widgets = {}  # {'widget name': tw.api.Widget}
        moksha.common.utils.menus = {}  # {'menu name': moksha.api.menus.MokshaMenu}
        self.engines = {}  # {'app name': sqlalchemy.engine.base.Engine}

        self.load_paths()
        self.load_configs()
        self.load_widgets()
        self.load_applications()
        self.load_models()
        self.load_menus()
        self.load_root()

    def __call__(self, environ, start_response):
        self.register_livewidgets(environ)
        return self.application(environ, start_response)

    def register_livewidgets(self, environ):
        """ Register the `moksha.livewidgets` dictionary.

        This is a per-request StackedObjectProxy that is used by the
        LiveWidgets to register their own topic callbacks.  The Moksha Live
        Socket then handles subscribing widgets to their appropriate topics,
        decoding the incoming JSON data, and dispatching messages to them as
        they arrive.
        """
        environ['paste.registry'].register(moksha.wsgi.lib.utils.livewidgets, {
            'onopen': [],
            'onclose': [],
            'onerror': [],
            'onerrorframe': [],
            'onconnectedframe': [],
            'onmessageframe': defaultdict(list)  # {topic: [js_callback,]}
        })

    def load_paths(self):
        """ Load the names and paths of all moksha applications and widgets.

        We must do this before actually loading the widgets or applications, to
        ensure that we parse and load each of their configuration files
        beforehand.
        """
        for app_entry in pkg_resources.iter_entry_points(APPS):
            if app_entry.name in moksha.common.utils._apps:
                raise MokshaException('Duplicate application name: %s' %
                                      app_entry.name)
            app_path = app_entry.dist.location
            moksha.common.utils._apps[app_entry.name] = {
                    'name': app_entry.name,
                    'project_name': app_entry.dist.project_name,
                    'path': app_path,
                    }
        for widget_entry in pkg_resources.iter_entry_points(WIDGETS):
            if widget_entry.name in moksha.common.utils._widgets:
                raise MokshaException('Duplicate widget name: %s' %
                                      widget_entry.name)
            widget_path = widget_entry.dist.location
            moksha.common.utils._widgets[widget_entry.name] = {
                    'name': widget_entry.name,
                    'project_name': widget_entry.dist.project_name,
                    'path': widget_path,
                    }

    def load_applications(self):
        log.info('Loading moksha applications')
        for app_entry in pkg_resources.iter_entry_points(APPS):
            log.info('Loading %s application' % app_entry.name)
            app_class = app_entry.load()
            app_path = app_entry.dist.location
            app_name = getattr(app_class, 'name', app_entry.name)
            if isclass(app_class):
                app_class = app_class()
            moksha.common.utils._apps[app_entry.name].update({
                    'name': app_name,
                    'controller': app_class,
                    'path': app_path,
                    'model': None,
                    })
            try:
                # Try to import the 'model' module alongside its 'controllers'
                module = '.'.join(app_class.__module__.split('.')[:-2] +
                                  ['model'])
                model = __import__(module, globals(), locals(),
                                   [app_entry.name])
                moksha.common.utils._apps[app_entry.name]['model'] = model
            except ImportError, e:
                log.debug("Cannot find application model: %r" % module)

    def load_widgets(self):
        """ Load widgets from entry points. """

        log.info('Loading moksha widgets')

        import tw2.core.widgets
        from moksha.wsgi.widgets.api.live import LiveWidgetMeta

        def is_live(widget):
            return isinstance(widget, LiveWidgetMeta)

        for widget_entry in pkg_resources.iter_entry_points(WIDGETS):
            log.info('Loading %s widget' % widget_entry.name)

            widget_class = widget_entry.load()
            if isinstance(widget_class, types.FunctionType):
                widget_class = widget_class(config=self.config)

            widget_path = widget_entry.dist.location
            moksha.common.utils._widgets[widget_entry.name] = {
                    'name': getattr(widget_class, 'name', widget_entry.name),
                    'widget': widget_class,
                    'path': widget_path,
                    'live': is_live(widget_class),
                    }

    def load_menus(self):
        log.info('Loading moksha menus')
        for menu_entry in pkg_resources.iter_entry_points(MENUS):
            log.info('Loading %s menu' % menu_entry.name)
            menu_class = menu_entry.load()
            menu_path = menu_entry.dist.location
            moksha.common.utils.menus[menu_entry.name] = menu_class(menu_entry.name)

    def load_configs(self):
        """ Load the configuration files for all applications.

        Here we iterate over all applications, loading their configuration
        files and merging their [DEFAULT] configuration into ours.  This
        requires that applications do not have conflicting configuration
        variable names.  To mitigate this, applications should use some basic
        variable namespacing, such as `myapp.myvariable = myvalue`.

        We first make sure to load up Moksha's configuration, for the cases
        where it is being run as WSGI middleware in a different environment.

        """
        apps = []
        loaded_configs = []
        conf_d = '/etc/moksha/conf.d/%s/'

        moksha_config_path = get_moksha_config_path()
        if moksha_config_path:
            moksha_config_path = os.path.dirname(moksha_config_path)
            apps = [{'path': moksha_config_path}]

        apps += moksha.common.utils._apps.values()
        for app in apps:
            for configfile in ('production.ini', 'development.ini'):
                for path in (app['path'], conf_d % app.get('project_name')):
                    confpath = os.path.join(path, configfile)
                    if os.path.exists(confpath):
                        conf = appconfig('config:' + confpath)
                        if app.get('name'):
                            moksha.common.utils._apps[app['name']]['config'] = conf
                        if confpath in loaded_configs:
                            continue
                        log.info('Loading configuration: %s' % confpath)
# This is leftover from the days of using paste.deploy.appconfig.  Is anything
# using this?
#                       for entry in conf.global_conf:
#                           if entry.startswith('_'):
#                               continue
#                           if entry in config:
#                               log.warning('Conflicting variable: %s' % entry)
#                               continue
#                           else:
#                               config[entry] = conf.global_conf[entry]
#                               log.debug('Set `%s` in global config' % entry)
                        loaded_configs.append(confpath)
                        break

    def load_models(self):
        """ Setup the SQLAlchemy database models for all moksha applications.

        This method first looks to see if your application has a
        ``sqlalchemy.url`` set in it's configuration file, and will create a
        SQLAlchemy engine with it.  If it does not exist, Moksha will create an
        engine for your application based on the ``app_db`` configuration,
        which defaults to ``sqlite:///$APPNAME.db``.

        It will then bind the engine to your model's
        :class:`sqlalchemy.MetaData`, and initialize all of your tables,
        if they don't already exist.

        """
        for name, app in moksha.common.utils._apps.items():
            sa_url = app.get('config', {}).get('sqlalchemy.url', None)
            app_db = self.config.get('app_db', 'sqlite:///%s.db')
            if sa_url:
                if app['config']['__file__'] == get_moksha_config_path():
                    # Moksha's apps don't specify their own SA url
                    self.engines[name] = create_engine(app_db % name)
                else:
                    # App has specified its own engine url
                    self.engines[name] = create_engine(sa_url)

            # If a `model` module exists in the application, call it's
            # `init_model` method,and bind the engine to it's `metadata`.
            if app.get('model'):
                if not sa_url:
                    self.engines[name] = create_engine(app_db % name)
                log.debug('Creating database engine for %s' % app['name'])
                app['model'].init_model(self.engines[name])
                app['model'].metadata.create_all(bind=self.engines[name])

    def load_root(self):
        """ Load the root controller.

        This allows developers to configure Moksha to directly hit their
        TurboGears controller or WSGI app.  You can also have the root of your
        website be a single widget.

        This is an example entry-point in your setup.py/pavement.py::

            [moksha.root]
            root = myproject.controllers.root:RootController

        """
        root = None
        for root_entry in pkg_resources.iter_entry_points(ROOT):
            log.info('Loading the root of the project: %r' %
                     root_entry.dist.project_name)
            if root_entry.name == 'root':
                root_class = root_entry.load()
                moksha.common.utils.root = root_class

               # TODO: support setting a root widget
               #if issubclass(root_class, Widget):
               #    widget = root_class(root_class.__name__)
               #    moksha.common.utils._widgets[root_entry.name] = {
               #        'name': getattr(root_class, 'name', widget_entry.name),
               #        'widget': widget,
               #        'path': root_entry.dist.location,
               #        }

               # TODO: handle root wsgi apps
            else:
                log.error('Ignoring [moksha.root] entry %r')
                log.error('Please expose at most 1 object on this entry-point,'
                          ' named "root".')


def make_moksha_middleware(app, config):
    if asbool(config.get('moksha.connectors', False)):
        raise NotImplementedError(
            "moksha.connectors has moved to fedora-community"
        )

    if asbool(config.get('moksha.extensionpoints', True)):
        from moksha.wsgi.middleware import MokshaExtensionPointMiddleware
        app = MokshaExtensionPointMiddleware(app, config)

    app = MokshaMiddleware(app, config)

    if asbool(config.get('moksha.csrf_protection', False)):
        raise NotImplementedError(
            "moksha.csrf_protection has been moved to python-fedora")

    if asbool(config.get('moksha.registry', True)):
        from paste.registry import RegistryManager
        app = RegistryManager(app)

    return app



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>
#          Ralph Bean <ralph.bean@gmail.com>

import moksha
import moksha.wsgi.lib.utils

from uuid import uuid4

from moksha.common.exc import MokshaException
from moksha.wsgi.widgets.api.stomp import (
    StompWidget, stomp_subscribe, stomp_unsubscribe)
from moksha.wsgi.widgets.api.amqp import (
    AMQPSocket, amqp_subscribe, amqp_unsubscribe)
from moksha.wsgi.widgets.api.websocket import (
    WebSocketWidget, websocket_subscribe, websocket_unsubscribe)

import tw2.core as twc
import tw2.core.params as pm
import tw2.core.widgets


class LiveWidgetMeta(tw2.core.widgets.WidgetMeta):
    pass


class LiveWidget(twc.Widget):
    """ A live streaming widget based on toscawidgets2

    This widget handles automatically subscribing your widget to any given
    topics, and registers all of the stomp callbacks.

    The basics of the LiveWidget::

        class MyLiveWidget(LiveWidget):
            topic = 'mytopic'
            onmessage = 'console.log(json)'
            template = 'mako:myproject.templates.mylivewidget'
    """
    __metaclass__ = LiveWidgetMeta

    backend = pm.Param('moksha livesocket backend to use',
                       default=twc.Required)
    topic = pm.Param('Topic to which this widget is subscribed')
    onmessage = pm.Param('js to execute on message received', default=None)

    def prepare(self):
        """ Get this widget ready for display

        Register this widget's message topic callbacks
        """
        if not hasattr(self, 'id'):
            self.id = str(uuid4())
            self.compound_id = self.id

        if not self.onmessage:
            raise MokshaException('%s must be provided an onmessage callback' %
                                  self.__class__.__name__)

        super(LiveWidget, self).prepare()

        if not self.topic:
            raise MokshaException('You must specify a `topic` to subscribe to')

        topics = isinstance(self.topic, list) and self.topic or [self.topic]

        backend_lookup = {
            'stomp': StompWidget.callbacks,
            'amqp': AMQPSocket.callbacks,
            'websocket': WebSocketWidget.callbacks,
        }
        callbacks = backend_lookup[self.backend]

        for callback in callbacks:
            if callback == 'onmessageframe':
                for topic in topics:
                    cb = getattr(self, 'onmessage').replace('${id}', self.id)
                    moksha.wsgi.lib.utils.livewidgets[callback][topic].append(cb)
            elif callback in ['onconnectedframe', 'onopen']:
                moksha.wsgi.lib.utils.livewidgets[callback].append(
                    self.subscribe_topics(topics))
            elif getattr(self, callback, None):
                moksha.wsgi.lib.utils.livewidgets[callback].append(
                    getattr(self, callback))

    @classmethod
    def get_topics(cls):
        topics = []
        for key in ('topic', 'topics'):
            if hasattr(cls, key):
                topic = getattr(cls, key)
                if topic:
                    if isinstance(topic, basestring):
                        map(topics.append, topic.split())
                    else:
                        topics += topic
        return topics

    def subscribe_topics(self, topics):
        backend_lookup = {
            'stomp': stomp_subscribe,
            'amqp': amqp_subscribe,
            'websocket': websocket_subscribe,
        }
        try:
            return backend_lookup[self.backend](topics)
        except KeyError:
            raise MokshaException("Unknown `moksha.livesocket.backend` %r. "
                                  "Valid backends are currently %s" % (
                                      self.backend,
                                      ", ".join(backend_lookup.keys())
                                  ))

    def unsubscribe_topics(self, topics):
        backend_lookup = {
            'stomp': stomp_unsubscribe,
            'amqp': amqp_unsubscribe,
            'websocket': websocket_unsubscribe,
        }
        try:
            return backend_lookup[self.backend](topics)
        except KeyError:
            raise MokshaException("Unknown `moksha.livesocket.backend` %r. "
                                  "Valid backends are currently %s" % (
                                      self.backend,
                                      ", ".join(backend_lookup.keys())
                                  ))


# Moksha Topic subscription handling methods
subscribe_topics = LiveWidget.subscribe_topics
unsubscribe_topics = LiveWidget.unsubscribe_topics



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

import tw2.core as twc
import tw2.jquery

orbited_host = twc.Required
orbited_port = twc.Required
orbited_scheme = twc.Required


class OrbitedWidget(twc.Widget):
    onopen = twc.Param("A javascript callback for when the connection opens",
                       default=twc.JSSymbol(src="function(){}"))
    onread = twc.Param("A javascript callback for when new data is read",
                       default=twc.JSSymbol(src="function(){}"))
    onclose = twc.Param("A javascript callback for when the connection closes",
                        default=twc.JSSymbol(src="function(){}"))
    orbited_port = twc.Param("Orbited port", default=orbited_port)
    orbited_host = twc.Param("Orbited host", default=orbited_host)
    orbited_scheme = twc.Param("Orbited scheme", default=orbited_scheme)
    template = "mako:moksha.wsgi.widgets.api.orbited.templates.orbited"

    def prepare(self):
        orbited_url = '%s://%s:%s' % (
            self.orbited_scheme,
            self.orbited_host,
            self.orbited_port,
        )
        orbited_js = twc.JSLink(link=orbited_url + '/static/Orbited.js',
                                resources=[tw2.jquery.jquery_js])
        self.resources.append(orbited_js)



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

import tw2.jqplugins.flot
import tw2.excanvas

from moksha.wsgi.widgets.api import LiveWidget


class LiveFlotWidget(LiveWidget):
    """ A live graphing widget """
    topic = None
    params = ['id', 'data', 'options', 'height', 'width', 'onmessage']
    onmessage = '$.plot($("#${id}"),json[0]["data"],json[0]["options"])'
    template = "mako:moksha.wsgi.widgets.api.flot.templates.flot"
    resources = [
        tw2.jqplugins.flot.flot_js,
        tw2.excanvas.excanvas_js
    ]
    height = '250px'
    width = '390px'
    options = {}
    data = [{}]



def global_resources():
    """ Returns a rendered Moksha Global Resource Widget.

    This widget contains all of the resources and widgets on the
    `moksha.global` entry-point.  To use it, simply do this at the bottom of
    your master template::

        ${tmpl_context.moksha_global_resources()}

    :Warning: It must be called at the *bottom* of your template.  It must be
              the last thing that executes.  It won't inject resources for
              widgets that get displayed *after* this function is called.

    Often, you want moksha's global resources to appear on every page.
    You can override the BaseController of your app in ``yourapp.lib.base`` and
    set the following inside the __call__ method::

        from moksha.wsgi.ext.turbogears import global_resources
        tmpl_context.moksha_global_resources = global_resources

    """
    import tg
    from moksha.wsgi.widgets.api.global_resources import global_resources as globs
    globs = globs(config=tg.config, request=tg.request, base_url=tg.url('/'))

    if tg.config.default_renderer == 'genshi':
        # There's Got To Be A Better Way!
        # TODO -- consider webhelpers.literal
        from genshi import unescape, Markup
        return Markup(unescape(Markup(globs.display())))
    elif tg.config.default_renderer == 'mako':
        return globs.display()
    else:
        # If this gets called, and explodes, then you need to add support
        # for your templating engine here.
        return globs.display()



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
:mod:`moksha.wsgi.widgets.api.websocket` - An "WebSocket" Live Moksha socket
==================================================================

.. moduleauthor:: Ralph Bean <rbean@redhat.com>
"""

import tw2.core as twc
from kitchen.iterutils import iterate

from moksha.wsgi.widgets.api.socket import AbstractMokshaSocket


def websocket_subscribe(topic):
    """ Return a javascript callback that subscribes to a given topic,
        or a list of topics.
    """
    sub = "moksha.topic_subscribe('%(topic)s');"
    return ''.join([sub % {'topic': t} for t in iterate(topic)])


def websocket_unsubscribe(topic):
    """ Return a javascript callback that unsubscribes to a given topic,
        or a list of topics.
    """
    return ""
    # TODO:
    #sub = "stomp.unsubscribe('%s');"
    #if isinstance(topic, list):
    #    sub = ''.join([sub % t for t in topic])
    #else:
    #    sub = sub % topic
    #return sub


class WebSocketWidget(AbstractMokshaSocket):
    __shorthand__ = 'WebSocket'

    ws_scheme = twc.Param(default=twc.Required)
    ws_host = twc.Param(default=twc.Required)
    ws_port = twc.Param(default=twc.Required)

    template = "mako:moksha.wsgi.widgets.api.websocket.templates.websocket"



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
:mod:`moksha.wsgi.widgets.source` -- Source Code Widget
==================================================

The SourceCodeWidget takes a Widget instance, and
returns the source code, in HTML.

.. moduleauthor:: Luke Macken <lmacken@redhat.com>
"""

import moksha.common.utils
import inspect

import tw2.core as twc

from twisted.python.reflect import namedAny
from pygments import highlight
from pygments.lexers import PythonLexer
from pygments.formatters import HtmlFormatter

class SourceCodeWidget(twc.Widget):
    widget = twc.Param("The name of the widget")
    module = twc.Param("Whether to display the entire module")
    source = twc.Param("Optional source code", default=None)
    code = twc.Param("The actual rendered source code")
    title = twc.Param("An optional title for the document", default='')

    template = "mako:moksha.wsgi.widgets.templates.source"

    container_options = {'width': 625, 'height': 675, 'title': 'View Source',
                         'icon': 'comment.png', 'top': 50, 'left': 300}
    hidden = True

    def prepare(self):
        super(SourceCodeWidget, self).prepare()
        title = self.widget.__class__.__name__
        if not self.source:
            try:
                self.widget = moksha.utils.get_widget(self.widget)
            except Exception, e:
                self.widget = namedAny(self.widget)
            if self.module:
                obj = namedAny(self.widget.__module__)
            else:
                obj = self.widget.__class__
            self.source = inspect.getsource(obj)
        html_args = {'full': True}
        if self.title:
            html_args['title'] = self.title
        self.code = highlight(self.source, PythonLexer(),
                              HtmlFormatter(**html_args))

code_widget = SourceCodeWidget(id='code_widget')



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

from random import random
from datetime import timedelta

from moksha.wsgi.widgets.api import LiveWidget
# TODO -- need to refactor this and move the livegraphproducer to a non-wsgi
# module.
#from moksha.hub.api.producer import PollingProducer

import tw2.core
import tw2.core.resources as res
import tw2.core.params as pm


class LiveGraphWidget(LiveWidget):
    """
    This is an example live graph widget based on Michael Carter's article
    "Scalable Real-Time Web Architecture, Part 2: A Live Graph with Orbited,
    MorbidQ, and js.io".

    http://cometdaily.com/2008/10/10/scalable-real-time-web-architecture-part-2-a-live-graph-with-orbited-morbidq-and-jsio
    """
    onconnectedframe = pm.Param('callback (put a description of what this is here...)')
    onmessageframe = pm.Param('callback (put a description here..)')
    topic = 'graph_demo'
    onmessage = 'modify_graph(bars, frame.body)'
    resources = [
        res.JSLink(filename='static/livegraph.js', modname=__name__),
        res.CSSLink(filename='static/livegraph.css', modname=__name__),
    ]
    template = '<div id="${id}" />'

    def prepare(self):
        super(LiveGraphWidget, self).prepare()
        self.add_call(tw2.core.JSFuncCall('init_graph', self.id))


#class LiveGraphProducer(PollingProducer):
#    """
#    This is the main data producer for our live graph widget.
#    """
#    frequency = timedelta(seconds=0.5)
#
#    data_vector_length = 10
#    delta_weight = 0.1
#    max_value = 400 # NB: this in pixels
#    data = [int(random()*max_value) for x in xrange(data_vector_length)]
#
#    def poll(self):
#        self.data = [
#            min(max(datum + (random() - .5) * self.delta_weight *
#                    self.max_value, 0),
#                self.max_value)
#            for
#            datum in self.data
#        ]
#        self.send_message('graph_demo', self.data)



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import tw2.core as twc

from moksha.wsgi.widgets.container import MokshaContainer


# set up the middleware for the tests
def request_local_tst():
    global _request_local, _request_id
    #    if _request_id is None:
    #        raise KeyError('must be in a request')
    if _request_local == None:
        _request_local = {}
    try:
        return _request_local[_request_id]
    except KeyError:
        rl_data = {}
        _request_local[_request_id] = rl_data
        return rl_data

twc.core.request_local = request_local_tst
_request_local = {}
_request_id = 'whatever'


class TestContainer:

    def setUp(self):
        twc.core.request_local = request_local_tst
        twc.core.request_local()['middleware'] = twc.make_middleware()
        self.w = MokshaContainer(id='test')

    def test_render_widget(self):
        assert 'Moksha Container' in self.w.display()

    def test_widget_content(self):
        """ Ensure we can render a container with another widget """

        class MyWidget(twc.Widget):
            template = "mako:moksha.wsgi.widgets.container.tests.templates.w"

        assert 'Hello World!' in self.w.display(
            content=MyWidget(id='mywidget'))

    def test_container_classes(self):
        rendered = self.w.display(**dict(skin3=True, stikynote=True,
                                         draggable=True, resizable=True))
        assert(
            'class="containerPlus draggable resizable"' in rendered,
            rendered
        )



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import tw2.core as twc
import tw2.jquery
import tw2.jqplugins.ui

from uuid import uuid4


moksha_ui_selectable_js = twc.JSLink(
    modname='moksha',
    filename='public/javascript/ui/moksha.ui.selectable.js',
    resources=[tw2.jqplugins.ui.jquery_ui_js])


class Selectable(twc.Widget):
    template = 'mako:moksha.wsgi.widgets.api.selectable.templates.selectable'
    resources = [moksha_ui_selectable_js]

    def prepare(self):
        super(Selectable, self).prepare()
        content_id = d.id + '-uuid' + str(uuid4())
        self.content_id = content_id

        self.add_call(
            tw2.jquery.jQuery("#%s" % self.content_id).moksha_selectable())



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
:mod:`moksha.wsgi.widgets.api.amqp` - An AMQP driven live Moksha socket
==================================================================

.. moduleauthor:: Luke Macken <lmacken@redhat.com>
"""

import tw2.core as twc

from moksha.wsgi.widgets.api.socket import AbstractMokshaSocket
from kitchen.iterutils import iterate

orbited_host = twc.Required
orbited_port = twc.Required
orbited_scheme = twc.Required

jsio_js = twc.JSLink(
    filename='static/jsio/jsio.js',
    modname=__name__)

amqp_resources = twc.DirLink(
    filename='static/',
    modname=__name__)


def amqp_subscribe(topic):
    """ Return a javascript callback that subscribes to a given topic,
        or a list of topics.
    """
    sub = """
        moksha.debug("Subscribing to the '%(topic)s' topic");
        var receiver = moksha_amqp_session.receiver('amq.topic/%(topic)s')
        receiver.onReady = raw_msg_callback;
        receiver.capacity(0xFFFFFFFF);
    """
    return ''.join([sub % {'topic': t} for t in iterate(topic)])


def amqp_unsubscribe(topic):
    """ Return a javascript callback that unsubscribes to a given topic,
        or a list of topics.
    """
    return ""
    # TODO:
    #sub = "stomp.unsubscribe('%s');"
    #if isinstance(topic, list):
    #    sub = ''.join([sub % t for t in topic])
    #else:
    #    sub = sub % topic
    #return sub


class AMQPSocket(AbstractMokshaSocket):
    __shorthand__ = "AMQP Socket"

    resources = AbstractMokshaSocket.resources + \
            [amqp_resources, jsio_js]

    orbited_host = twc.Param(default=orbited_host)
    orbited_port = twc.Param(default=orbited_port)
    orbited_scheme = twc.Param(default=orbited_scheme)

    moksha_domain = twc.Param(twc.Required)

    amqp_broker_host = twc.Param(default=twc.Required)
    amqp_broker_port = twc.Param(default=twc.Required)
    amqp_broker_user = twc.Param(default=twc.Required)
    amqp_broker_pass = twc.Param(default=twc.Required)

    template = "mako:moksha.wsgi.widgets.api.amqp.templates.amqp"

    def prepare(self):
        super(AMQPSocket, self).prepare()
        self.orbited_url = '%s://%s:%s' % (
            self.orbited_scheme,
            self.orbited_host,
            self.orbited_port,
        )



import tw2.core as twc
from tw2.jqplugins.gritter import gritter_resources, gritter_callback
from collections import defaultdict

from moksha.wsgi.widgets.moksha_js import moksha_js
import moksha.wsgi.lib.utils


class AbstractMokshaSocket(twc.Widget):
    """ Abstract socket widget from which the AMQPSocket, STOMPSocket and
    WebSocketSocket inherit.

    NOTE - this only exists for tw2.  I'm not bothering with refactoring tw1
    code now.
    """

    __shorthand__ = twc.Variable(default=None)

    resources = [moksha_js]
    topics = twc.Variable()

    hidden = twc.Variable(default=True)

    notify = twc.Param(default=twc.Required)
    reconnect_interval = twc.Param(default=twc.Required)

    callbacks = [
        "onopen",
        "onclose",
        "onerror",
        "onconnectedframe",
        "onmessageframe",
        "onerrorframe",
    ]

    onopen = twc.Param(default='')
    onclose = twc.Param(default='')
    onerror = twc.Param(default='')
    onerrorframe = twc.Param(default='')
    onconnectedframe = twc.Param(default='')

    # Used internally
    before_open = twc.Variable(default='')

    notifications = {
        'before_open': 'Attempting to connect Moksha Live Socket',
        'onconnectedframe': 'Moksha Live socket connected',  # AMQP, STOMP
        'onopen': 'Moksha Live socket connected',            # WebSocket
        'onclose': 'Moksha Live socket closed',
        'onerrorframe': 'Error with Moksha Live socket',
        'onerror': 'Error with Moksha Live socket',
    }

    def prepare(self):
        super(AbstractMokshaSocket, self).prepare()

        if not self.__shorthand__:
            raise ValueError("SocketWidget must declare __shorthand__")

        self.topics = []
        self.onmessageframe = defaultdict(str)

        if self.notify:
            self.resources += gritter_resources
            self.before_open = "$(%s);" % unicode(gritter_callback(
                title=self.__shorthand__,
                text=self.notifications['before_open'],
            ))

        for callback in self.callbacks:
            cbs = ''

            if self.notify and callback in self.notifications:
                cbs += "$(%s);" % unicode(gritter_callback(
                    title=self.__shorthand__,
                    text=self.notifications[callback]
                ))

            if self.reconnect_interval and callback is 'onclose':
                cbs += "setTimeout(setup_moksha_socket, %i)" % \
                        int(self.reconnect_interval)

            if len(moksha.wsgi.lib.utils.livewidgets[callback]):
                if callback == 'onmessageframe':
                    for topic in moksha.wsgi.lib.utils.livewidgets[callback]:
                        self.topics.append(topic)
                        for cb in moksha.wsgi.lib.utils.livewidgets[callback][topic]:
                            self.onmessageframe[topic] += '%s;' % unicode(cb)
                else:
                    for cb in moksha.wsgi.lib.utils.livewidgets[callback]:
                        if isinstance(cb, (twc.js_callback, twc.js_function)):
                            cbs += '$(%s);' % unicode(cb)
                        else:
                            cbs += unicode(cb)
            if cbs:
                setattr(self, callback, cbs)




import unittest
import moksha.wsgi.middleware
import webtest

from nose.tools import raises
from nose.tools import eq_


class TestMiddleware(unittest.TestCase):
    def setUp(self):
        def app(environ, start_response):
            start_response('200 OK', [('Content-Type', 'text/plain')])
            return ['Hello, world!\n']

        self.app = app

    def test_middleware_wrap(self):
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, {})

    def test_middleware_simple(self):
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, {})
        test_app = webtest.TestApp(app)
        response = test_app.get('/')
        eq_(response.status, '200 OK')

    @raises(KeyError)
    def test_no_registry(self):
        config = {'moksha.registry': False}
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, config)
        response = webtest.TestApp(app).get('/')

    def test_external_registry(self):
        config = {'moksha.registry': False}
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, config)
        from paste.registry import RegistryManager
        app = RegistryManager(app)
        response = webtest.TestApp(app).get('/')
        eq_(response.status, '200 OK')

    @raises(NotImplementedError)
    def test_connectors(self):
        config = {'moksha.connectors': True}
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, config)
        response = webtest.TestApp(app).get('/')

    @raises(NotImplementedError)
    def test_csrf(self):
        config = {'moksha.csrf_protection': True}
        app = moksha.wsgi.middleware.make_moksha_middleware(self.app, config)
        response = webtest.TestApp(app).get('/')



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: Luke Macken <lmacken@redhat.com>

import uuid
import tw2.core as twc
import tw2.jquery

from moksha.wsgi.widgets.api.live import LiveWidget, LiveWidgetMeta
from moksha.wsgi.widgets.api.live import subscribe_topics, unsubscribe_topics


container_js = twc.JSLink(
    filename='static/js/mbContainer.min.js',
    resources=[
        tw2.jquery.jquery_js,
        tw2.jqplugins.ui.jquery_ui_js,
    ],
    modname=__name__)
container_css = twc.CSSLink(
    filename='static/css/mbContainer.css',
    resources=[
        twc.DirLink(filename='static/css/elements', modname=__name__)
    ],
    modname=__name__)


class MokshaContainer(twc.Widget):
    template = 'mako:moksha.wsgi.widgets.container.templates.container'
    resources = [container_js, container_css]
    options = ['draggable', 'resizable']
    button_options = ['iconize', 'minimize', 'close']
    params = ['buttons', 'skin', 'height', 'width', 'left', 'top', 'id',
              'title', 'icon', 'content', 'widget_name', 'view_source', 'dock',
              'onResize', 'onClose', 'onCollapse', 'onIconize', 'onDrag',
              'onRestore'] + options[:]
    draggable = droppable = True
    resizable = False
    iconize = minimize = close = True
    hidden = True  # hide from the moksha menu
    content = ''  # either text, or a Widget instance
    widget_name = ''
    title = 'Moksha Container'
    skin = 'default'  # default, black, white, stiky, alert
    view_source = True
    dock = 'moksha_dock'
    icon = 'gears.png'

    # Pixel tweaking
    width = 450
    height = 500
    left = 170
    top = 125

    # Javascript callbacks
    onResize = "function(o){}"
    onClose = "function(o){}"
    onCollapse = "function(o){}"
    onIconize = "function(o){}"
    onDrag = "function(o){}"
    onRestore = "function(o){}"

    def prepare(self):
        super(MokshaContainer, self).prepare()
        if self.content and not isinstance(self.content,
                                           tw2.core.widgets.WidgetMeta):
            # This does not have to be cause for ultra-alarm.  We could take
            # care to render tw1 widgets inside tw2 widgets, but I'm throwing
            # hard errors here so I don't mess up jquery dependencies.
            raise ValueError("non-tw2 widget found inside tw2 container")

        # If we weren't passed a widget_name explicitly, then take a guess.
        if not getattr(self, 'widget_name') and hasattr(self.content, 'id'):
            self.widget_name = self.content.id

        content_args = getattr(self, 'content_args', {})
        if isinstance(self.content, (LiveWidget, LiveWidgetMeta)):
            topics = self.content.get_topics()
            # FIXME: also unregister the moksha callback functions.  Handle
            # cases where multiple widgets are listening to the same topics

            obj = self.content.req()

            if not isinstance(unsubscribe_topics(obj, topics), basestring):
                raise ValueError('wtf')

            self.onClose = "function(o){%s $(o).remove();}" % \
                unsubscribe_topics(obj, topics)
            self.onIconize = self.onCollapse = "function(o){%s}" % \
                unsubscribe_topics(obj, topics)
            self.onRestore = "function(o){%s}" % \
                subscribe_topics(obj, topics)

        if self.content:
            self.content = self.content.display(**content_args)

        for option in self.options:
            setattr(self, option, getattr(self, option, True) and option or '')

        self.buttons = ''
        for button in self.button_options:
            if getattr(self, button, True):
                self.buttons += '%s,' % button[:1]
        self.buttons = self.buttons[:-1]

        self.id = str(uuid.uuid4())

        self.elementsPath = '/tw2/resources/moksha.wsgi.widgets.container.container/static/css/elements/'
#        self.add_call(tw2.jquery.jQuery('#%s' % self.id).buildContainers({
#            'elementsPath':
#            'onClose': self.onClose,
#            'onResize': self.onResize,
#            'onCollapse': self.onCollapse,
#            'onIconize': self.onIconize,
#            'onDrag': self.onDrag,
#            'onRestore': self.onRestore,
#            }))


container = MokshaContainer(id='moksha_container')



import webtest

import moksha.common.testtools.utils as testutils

from moksha.wsgi.widgets.api.live import get_moksha_socket
from moksha.wsgi.middleware import make_moksha_middleware
from tw2.core import make_middleware as make_tw2_middleware


class TestClientSocketDumb:
    def _setUp(self):
        def kernel(config):
            def app(environ, start_response):
                start_response('200 OK', [('Content-Type', 'text/html')])
                socket = get_moksha_socket(config)
                return map(str, [socket.display()])

            app = make_moksha_middleware(app, config)
            app = make_tw2_middleware(app, config)
            app = webtest.TestApp(app)
            self.app = app

        for _setup, name in testutils.make_setup_functions(kernel):
            yield _setup, name

    def _tearDown(self):
        pass

    @testutils.crosstest
    def test_has_socket_str(self):
        targets = ['moksha_websocket', 'TCPSocket']
        response = self.app.get('/')
        assert(any([target in response for target in targets]))



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Authors: John (J5) Palmieri <johnp@redhat.com>

import logging
import pkg_resources

from webob import Request, Response

from moksha.common.exc import ApplicationNotFound, MokshaException

log = logging.getLogger(__name__)

import shlex
import os

class Chunk(object):
    def __init__(self):
        self.consumes = []
        self.info = ""
        self.code = ""

    def parse_consumes_field(self):
        start = self.info.find('consumes') + 8
        start = self.info.find('[',start)
        end = self.info.find(']', start)

        consumes = self.info[start + 1 : end]

        self.consumes = shlex.split(consumes)

class MokshaExtensionPointMiddleware(object):
    """
    A layer of WSGI middleware that is responsible for serving
    javascript extensions.

    If a request for an extension comes in (/moksha_extension/$extension_point),
    it will serve up any extensions that consume the extension point.

    This works by searching for all extensions of a given type and creating a
    javascript array pointing to their run functions.

    All extensions should contain at least a run function which will be passed
    the data fields specified by the extension point.  They should also return
    the HTML they wish to be embedded at the extension point. e.g.:

    function run(data) {
      // data will always contain a uid field and the HTML you return
      // will be wrapped in a div with that id
      return "<div>Hello</div>";
    };

    Extensions also need a header which describes the extension as such:

    info : {'consumes':['build_message'],
            'author': 'John (J5) Palmieri <johnp@redhat.com>',
            'version': '0.1',
            'name': 'Hello World Message'}

    """
    def __init__(self,
                 application,
                 config,
                 entry_point='moksha.extension_point',
                 test_dir=None):
        """
        :application: WSGI application to wrap
        :config: a dict of config values
        :extension_point: the python extry point which specifies modules to
                          scan for JavaScript extension_points
        :test_dir: a directory to scan for JavaScript extension_points which
                   are being tested before they are added to the module
        """

        log.info('Creating MokshaExtensionPointMiddleware')
        self.application = application

        # if debug is False condense javascript to optimize
        self.__debug = config.get('moksha.extension_points.debug')
        self.__extension_cache = {}
        self.application = application

        if test_dir:
            self.load_extension_dir(test_dir)

        for ep in pkg_resources.iter_entry_points(entry_point):
            mod = ep.load()
            dir = os.path.dirname(mod.__file__)
            self.load_extension_dir(dir)

    def chunk_code(self, js, filename):
        start = js.find('{')
        end = 0
        chunks = []
        length = len(js)

        while (start != -1):
            count = 1
            i = start + 1
            pull_info = False
            pull_info_start = 0
            pull_info_end = 0

            while(count != 0 and i < length):
                if js[i] == '{':
                    count += 1
                elif js[i] == '}':
                    count -= 1
                elif count==1 and js[i]=='i' and js.find('info',i) == i:
                    if pull_info_start != 0:
                        print "ERROR: Unexpected Info Field : extension file %s has more than on info field per extension (ignoring file)" % filename
                        return None

                    # put us on info duty
                    pull_info = True

                if pull_info:
                    if pull_info_start == 0:
                        if count > 1:
                            pull_info_start = i
                    else:
                        if count == 1:
                            pull_info_end = i
                            pull_info = False

                i += 1

            end = i

            c = Chunk()
            c.code = js[start:end]
            c.info = js[pull_info_start:pull_info_end + 1]

            c.parse_consumes_field()
            chunks.append(c)

            if i >= length:
                print "ERROR: Unexpected EOF: extension file %s has mismatched braces (ignoring file)" % filename
                return None

            start = js.find('{', end)

        return chunks

    def load_extension(self, file):
        log.info("Loading JavaScript extension %s" % file)
        f = open(file,'r')
        js = f.read()

        chunks = self.chunk_code(js, file)
        if not chunks:
            if chunks != None:
                log.warning("No Content : extension file %s doesn't have any valid extensions (ignoring file)" % filename)

            return

        for c in chunks:
            for exttype in c.consumes:
                code = self.__extension_cache.get(exttype, [])
                #TODO: run through optimizer which strips whitespace
                code.append(c.code)
                self.__extension_cache[exttype] = code

    def load_extension_dir(self, dir):
        for root, dirs, files in os.walk(dir):
            for name in files:
                if name.endswith('js'):
                    path = os.path.join(root, name)
                    self.load_extension(path)

        # compile lists in the cache down to a string so we don't have to
        # process it on each request
        for key, value in self.__extension_cache.iteritems():
            s = '[' + ','.join(value) + ']'
            self.__extension_cache[key] = s

    def strip_script(self, environ, path):
        # Strips the script portion of a url path so the middleware works even
        # when mounted under a path other than root
        if path.startswith('/') and 'SCRIPT_NAME' in environ:
            prefix = environ.get('SCRIPT_NAME')
            if prefix.endswith('/'):
                prefix = prefix[:-1]

            if path.startswith(prefix):
                path = path[len(prefix):]

        return path

    def __call__(self, environ, start_response):

        request = Request(environ)

        path = self.strip_script(environ, request.path)
        if path.startswith('/moksha_extension_point'):
            exttype = request.params.get('exttype')
            if not exttype:
                response = Response('')
            else:
                extensions_data = self.__extension_cache.get(exttype, "")
                extensions_str = ','.join(extensions_data)

                script = 'var moksha_loaded_extensions ='
                script += extensions_data
                script += ';'
                # now run the deferred extensions queued up while the scripts
                # were being downloaded

                script += 'moksha.extensions._extension_cache["' + exttype +'"] = moksha_loaded_extensions;'
                script += 'var deferred=moksha.extensions._extension_deferred["' + exttype +'"];'
                script += 'var d=deferred.shift();'
                script += 'while(d){'
                script +=   'moksha.extensions.run_extensions(moksha_loaded_extensions, d);'
                script +=   'd = deferred.shift();'
                script += '}'

                response = Response(script)
        else:
            response = request.get_response(self.application)

        return response(environ, start_response)



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
:mod:`moksha.wsgi.widgets.api.buttons` - Fancy Button CSS
====================================================

This module contains a ToscaWidget for the mbButtons
project::

    http://www.open-lab.com/mb.ideas/index.html#mbButtons

.. moduleauthor:: Luke Macken <lmacken@redhat.com>
"""

import tw2.core as twc


static_images = twc.DirLink(
    filename='static/images/',
    modname=__name__)

buttons_css = twc.CSSLink(
    filename='static/buttons.css',
    resources=[static_images],
    media='all', modname=__name__)



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import sys

# This is required (oddly) to fix a python 2.7 bug with nose tests.
try:
    import multiprocessing
    import logging
except Exception:
    pass

tests_require = [
    'nose',
    'mock',
    'webtest',
    'bunch',
]

if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
    tests_require.extend([
        'unittest2',
    ])

setup(
    name='moksha.wsgi',
    version='1.2.4',
    description='WSGI components for Moksha.',
    author='Luke Macken, John (J5) Palmieri, Mairin Duffy, and Ralph Bean',
    author_email='',
    url='http://moksha.fedorahosted.org',
    install_requires=[
        "moksha.common>=1.0.6",
        "tw2.core",
        "tw2.forms",
        "tw2.jquery",
        "tw2.jqplugins.ui",
        "tw2.jqplugins.flot",
        "tw2.jqplugins.gritter",
        "tw2.jit",
        "tw2.excanvas",
        "paste",

        # Needed for the "paster moksha" commands
        # XXX -- potentially move this to its own moksha.devtools package?
        "PasteScript",
        "tempita",

        # Needed for the source code widget.
        "pygments",
    ],
    packages=find_packages(exclude=['ez_setup']),
    include_package_data=True,
    test_suite='nose.collector',
    namespace_packages=['moksha'],
    tests_require=tests_require,
    message_extractors = {'moksha': [
            ('**.py', 'python', None),
            ('templates/**.mak', 'mako', None),
            ('templates/**.html', 'genshi', None),
            ('public/**', 'ignore', None)]},

    entry_points="""
    # Pulls in all widgets.  Used by the `archive_moksha_resources` command.
    [toscawidgets.widgets]
    widgets = moksha.wsgi.widgets.all

    # Pulls in all widgets.  Used by the `archive_moksha_resources` command.
    [tw2.widgets]
    widgets = moksha.wsgi.widgets.all
    moksha_js = moksha.wsgi.widgets.moksha_js

    [distutils.commands]
    archive_moksha_resources = moksha.wsgi.distutils.command:archive_moksha_resources

    [moksha.widget]
    code_widget = moksha.wsgi.widgets.source:code_widget
    moksha_socket = moksha.wsgi.widgets.api.live:get_moksha_socket

    [moksha.global]
    moksha_socket = moksha.wsgi.widgets.api.live:get_moksha_socket

    [paste.filter_app_factory]
    middleware = moksha.wsgi.middleware:make_moksha_middleware

    [paste.global_paster_command]
    moksha = moksha.wsgi.pastetemplate:MokshaQuickstartCommand

    [paste.paster_create_template]
    moksha.master = moksha.wsgi.pastetemplate:MokshaMasterTemplate
    moksha.livewidget = moksha.wsgi.pastetemplate:MokshaLiveWidgetTemplate
    moksha.stream = moksha.wsgi.pastetemplate:MokshaStreamTemplate
    moksha.consumer = moksha.wsgi.pastetemplate:MokshaConsumerTemplate
    moksha.controller = moksha.wsgi.pastetemplate:MokshaControllerTemplate

    """,

)



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import pkg_resources
import types

from paste.deploy.converters import asbool

import tw2.core as twc
import tw2.core.widgets

from moksha.wsgi.widgets.moksha_js import (
    moksha_js, moksha_extension_points_js,
)
from moksha.wsgi.widgets.api.socket import AbstractMokshaSocket

log = logging.getLogger(__name__)


class GlobalResourceInjectionWidget(twc.Widget):
    """
    This widget will pull in all JSLink, CSSLink, and Widget resources that
    are listed on the `[moksha.global]` entry-point.

    :Note: Global Widget injection will only work when the global_resource
           widget is actually rendered in the template.  Otherwise, only JS
           and CSS resources will get injected.  Moksha's index.mak template
           handles this for us, otherwise you can import the `global_resources`
           widget from this module and do it yourself.
    """
    resources = [moksha_js]
    children = []
    css = []
    template = "mako:moksha.wsgi.widgets.api.global_resources.templates.global"

    base_url = '/'
    user_id = ''
    debug = twc.Param(default=False)
    profile = twc.Param(default=False)
    extensionpoints = twc.Param(default=False)

    base_url = twc.Param(default='/')

    config = twc.Param("Configuration dict.", default=twc.Required)
    request = twc.Param("The request.", default=twc.Required)


    @property
    def c(self):
        """ Synonym for the 'c' property for backwards compat with tw1 """
        return self.children

    def prepare(self):
        super(GlobalResourceInjectionWidget, self).prepare()

        for required in ['config', 'request']:
            if not getattr(self, required, None):
                raise ValueError("GlobalResources must be given a %r." %
                                 required)

        live = asbool(self.config.get('moksha.livesocket', True))

        for widget_entry in pkg_resources.iter_entry_points('moksha.global'):
            log.info('Loading global resource: %s' % widget_entry.name)
            loaded = widget_entry.load()
            if isinstance(loaded, types.FunctionType):
                loaded = loaded(self.config)
            if isinstance(loaded, twc.JSLink):
                self.resources.append(loaded)
            elif isinstance(loaded, twc.CSSLink):
                self.resources.append(loaded)
            elif isinstance(loaded, tw2.core.widgets.WidgetMeta):
                if loaded in self.children:
                    log.debug("Skipping duplicate global widget: %s" %
                              widget_entry.name)
                else:
                    if issubclass(loaded, AbstractMokshaSocket) and not live:
                        log.debug('Moksha Live Socket disabled in config')
                        continue

                    if issubclass(loaded, AbstractMokshaSocket) and any([
                        issubclass(c, AbstractMokshaSocket)
                        for c in self.children
                    ]):
                        log.debug('Not injecting two Moksha Live Sockets')
                        continue

                    self.children.append(loaded)
            else:
                raise Exception("Unknown global resource: %s.  Should be "
                                "either a JSLink or CSSLink." %
                                widget_entry.name)

        if self.extensionpoints:
            self.resources.append(moksha_extension_points_js)

        identity = self.request.environ.get('repoze.who.identity')
        if identity:
            self.user_id = identity.get('user_id', '')


global_resources = GlobalResourceInjectionWidget



# This file is part of Moksha.
# Copyright (C) 2008-2010  Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from paste.deploy.converters import asbool

from live import (
    LiveWidget, LiveWidgetMeta,
    subscribe_topics, unsubscribe_topics
)

from moksha.wsgi.widgets.api.stomp import StompWidget
from moksha.wsgi.widgets.api.amqp import AMQPSocket
from moksha.wsgi.widgets.api.websocket import WebSocketWidget

def _make_kwargs(mappings, config):
    return dict([
        (
            d['left_key'],
            d.get('formatter', lambda o: o)(
                config.get(d['right_key'], d['default'])
            )
        ) for d in mappings
    ])

def get_moksha_socket(config):
    livesocket_backend = \
            config.get('moksha.livesocket.backend', 'stomp').lower()

    # This is a mapping of config mappings to widget attributes for the three types
    # of socket widgets: stomp, amqp, and websocket.
    global_mappings = [
        dict(
            left_key='notify',
            right_key='moksha.socket.notify',
            default=True,
            formatter=asbool,
        ),
        dict(
            left_key='reconnect_interval',
            right_key='moksha.socket.reconnect_interval',
            default=None,
        ),
    ]
    orbited_mappings = [
        dict(
            left_key='orbited_host',
            right_key='orbited_host',
            default='localhost',
        ),
        dict(
            left_key='orbited_port',
            right_key='orbited_port',
            default='9000',
        ),
        dict(
            left_key='orbited_scheme',
            right_key='orbited_scheme',
            default='http',
        ),
    ]
    if livesocket_backend == 'stomp':
        mappings = global_mappings + orbited_mappings + [
            dict(
                left_key='stomp_broker',
                right_key='stomp_broker',
                default='localhost',
            ),
            dict(
                left_key='stomp_port',
                right_key='stomp_port',
                default='61613',
            ),
            dict(
                left_key='stomp_user',
                right_key='stomp_user',
                default='guest',
            ),
            dict(
                left_key='stomp_pass',
                right_key='stomp_pass',
                default='guest',
            ),
        ]
        socket_class = StompWidget
    elif livesocket_backend == 'amqp':
        mappings = global_mappings + orbited_mappings + [
            dict(
                left_key='moksha_domain',
                right_key='moksha.domain',
                default='localhost',
            ),
            dict(
                left_key='amqp_broker_host',
                right_key='amqp_broker_host',
                default='localhost',
            ),
            dict(
                left_key='amqp_broker_port',
                right_key='amqp_broker_port',
                default='5672',
            ),
            dict(
                left_key='amqp_broker_user',
                right_key='amqp_broker_user',
                default='guest',
            ),
            dict(
                left_key='amqp_broker_pass',
                right_key='amqp_broker_pass',
                default='guest',
            ),
        ]
        socket_class = AMQPSocket
    elif livesocket_backend == 'websocket':
        mappings = global_mappings + [
            dict(
                left_key='ws_scheme',
                right_key='moksha.livesocket.websocket.scheme',
                default='ws',
            ),
            dict(
                left_key='ws_host',
                right_key='moksha.livesocket.websocket.host',
                default='localhost',
            ),
            dict(
                left_key='ws_port',
                right_key='moksha.livesocket.websocket.port',
                default='9998',
            ),
        ]
        socket_class = WebSocketWidget
    else:
        raise Exception(
            "Unknown `moksha.livesocket.backend` %r.  Available backends: "
            "stomp, amqp, websocket" % livesocket_backend)

    moksha_socket = socket_class(**_make_kwargs(mappings, config))

    return moksha_socket
